<!DOCTYPE HTML PUBLIC "-//W3C//Dtd HTML 4.01 Transitional//EN">
<html>
<head>
<title>Ganglia Cluster Toolkit:: {page_title}</title>
<meta http-equiv="Content-type" content="text/html; charset=utf-8">
<meta http-equiv="refresh" content="300">
<link rel="stylesheet" href="./styles.css" type="text/css" />
</head>
<body style="background-color: #ffffff;">

<form action="{page}" method="GET" name="ganglia_form">

  <table width="100%" cellpadding="8" cellspacing="0" border="0">
  <tr>
     <td>
     {node_menu}
     </td>
     <td>
       <input type="submit" value="Get Fresh Data" />
     </td>
  </tr>
  </table>

<hr size="1" noshade />

