
body.night {
  background-color: #000;
}

div#title {
  margin: 20px 0% 0px;
}

#title span {
  font-size: 48px;
  font-family: Helvetica;
  font-weight: bold;
}

.night #title span {
  color: #fff;
  opacity: 0.9;
}

#toolbar {
  right: 0;
  float: right;
}

#title ul {
  list-style-type: none;
}

#title ul.timepanel {
  float: left;
}

#title ul.toggle {
  float: right;
}

#title ul li {
  float: left;
}

#title ul li.timepanel {
  padding: 10px 0;
  width: 90px;
  text-align: center;
  background-color: #999;
}

#title ul li.timepanel.live {
  width: 120px;
}

#title ul li.timepanel.night {
  background-color: #333;
}

#title ul li.timepanel.selected {
  background-color: #333;
  color: #fff;
}

#title ul li.timepanel.selected.night {
  background-color: #999;
}

#title ul li.timepanel a {
  font-family: Helvetica;
  font-size: 18px;
  font-weight: bold;
  color: #fff;
  text-decoration: none;
}

#title ul li.toggle-nonum, #title ul li.toggle-night {
  margin-left: 20px;
}

#title ul li a img {
  opacity: 0.8;
}

#title p {
  margin-left: 10px;
  font-size: 20px;
  font-family: Helvetica;
  color: #999;
}

div#main {
  margin: 30px 0% 0px;
  text-align: center;
}

div#nav {
  margin: 30px 5% 0px;
}

#nav ul {
  margin-left: 10px;
  list-style-type: none;
  padding: 0;
}

#nav li {
  padding-bottom: 3px;
}

#nav li a {
  font-size: 20px;
  font-family: Helvetica;
  text-decoration: none;
  color: #999;
}

#nav li a:hover {
  color: red;
}

#graph {
  position: relative;
  float: left;
  padding: 10px;
  margin: 10px;
}

#overlay-name {
  position: absolute;
  font-size: 24px;
  font-family: Helvetica;
  text-align: left;
  width: 150px;
  left: 20px;
  bottom: 20px;
  z-index: 200;
}

#overlay-number {
  position: absolute;
  font-size: 48px;
  font-family: Helvetica;
  font-weight: bold;
  right: 20px;
  bottom: 20px;
  z-index: 200;
}

#overlay-name.night, #overlay-number.night {
  color: #fff;
  opacity: 1.0;
}

#overlay-number.nonum {
  opacity: 0.0;
}

#overlay-number span.error {
  color: red;
}

span.unit {
  font-size: 24px;
}

div.x_label {
  font-size: 13px;
  font-family: Helvetica;
}

div.item {
  font-size: 12px;
  font-family: Helvetica;
}

